import threading

from diagnostic import Diagnostic
from earthdiagnostics import Utils, cdftools, TempFile
from autosubmit.config.log import Log
import os


class MixedLayerSaltContent(Diagnostic):

    def __init__(self, data_manager, startdate, member, chunk, lock):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.required_vars = ['so', 'mlotst']
        self.generated_vars = ['scvertsum']
        self.lock = lock

    @classmethod
    def generate_jobs(cls, diags, options):
        lock = threading.Lock()
        job_list = list()
        for startdate in diags.startdates:
            for member in diags.members:
                for chunk in range(1, diags.chunks + 1):
                    job_list.append(MixedLayerSaltContent(diags.datamanager, startdate, member, chunk, lock))
        return job_list

    def compute(self):
        """
        Compute mixed layer heat and salt content

        Created in February 2012      Author : vguemas@ic3.cat

        :return:
        """
        nco = Utils.nco
        cdo = Utils.cdo
        salinity_file = self.data_manager.get_file('ocean', 'so', self.startdate, self.member, self.chunk)
        mlotst_file = self.data_manager.get_file('ocean', 'mlotst', self.startdate, self.member, self.chunk)

        Utils.nco.ncks(input=mlotst_file, output=salinity_file, options='-A -v mlotst')

        ntime = int(cdo.ntime(input=salinity_file)[0])
        files = list()

        for time in range(ntime):
            Log.info('Running time {0}', time)
            temp = TempFile.get()
            temp2 = TempFile.get()
            nco.ncks(input=salinity_file, output=temp, options='-O -d time,{0}'.format(time))
            Log.info('Computing salt content')
            cdftools.run('cdfmxlsaltc', input=temp, output=temp2)
            os.remove(temp)
            files.append(temp2)

        temp = TempFile.get()
        cdo.cat(input=' '.join(files), output=temp,)
        nco.ncks(input=salinity_file, output=temp, options='-A -v time')

        for temp_file in files:
            os.remove(temp_file)
        os.remove(salinity_file)

        Utils.rename_variables(temp, {'x': 'i', 'y': 'j', 'somxlsaltc': 'scvertsum'}, False, True)
        Utils.setminmax(temp, 'scvsum')
        self.data_manager.send_file(temp, 'ocean', 'scvsum', self.startdate, self.member, self.chunk)


